const struct TVertex cvc[8]=
{
 {-1.0F,-1.0F,-1.0F},
 {-1.0F, 1.0F,-1.0F},
 { 1.0F, 1.0F,-1.0F},
 { 1.0F,-1.0F,-1.0F},
 { 1.0F,-1.0F, 1.0F},
 { 1.0F, 1.0F, 1.0F},
 {-1.0F, 1.0F, 1.0F},
 {-1.0F,-1.0F, 1.0F},
};

struct TVertex cv[8];

void Quad(
          float x0,float y0,float z0,float u0,float v0,
          float x1,float y1,float z1,float u1,float v1,
          float x2,float y2,float z2,float u2,float v2,
          float x3,float y3,float z3,float u3,float v3
         )
{
 Triangle(
          x0*MX,y0*MY,z0+OZ,u0,v0, //0
          x1*MX,y1*MY,z1+OZ,u1,v1, //1
          x2*MX,y2*MY,z2+OZ,u2,v2  //2
         );
 Triangle(
          x2*MX,y2*MY,z2+OZ,u2,v2, //2
          x3*MX,y3*MY,z3+OZ,u3,v3, //3
          x0*MX,y0*MY,z0+OZ,u0,v0  //0
         );
}

void Cube(void)
{
 Texture_Index=(u32*)TEXTURE;
 Quad(
      cv[0].X,cv[0].Y,cv[0].Z,0.0F,MaxV, //0
      cv[1].X,cv[1].Y,cv[1].Z,0.0F,0.0F, //1
      cv[2].X,cv[2].Y,cv[2].Z,MaxU,0.0F, //2
      cv[3].X,cv[3].Y,cv[3].Z,MaxU,MaxV  //3
     );

 Texture_Index+=MaxU*MaxV;
 Quad(
      cv[4].X,cv[4].Y,cv[4].Z,0.0F,MaxV, //4
      cv[5].X,cv[5].Y,cv[5].Z,0.0F,0.0F, //5
      cv[6].X,cv[6].Y,cv[6].Z,MaxU,0.0F, //6
      cv[7].X,cv[7].Y,cv[7].Z,MaxU,MaxV  //7
     );

 Texture_Index+=MaxU*MaxV;
 Quad(
      cv[3].X,cv[3].Y,cv[3].Z,0.0F,MaxV, //3
      cv[4].X,cv[4].Y,cv[4].Z,0.0F,0.0F, //4
      cv[7].X,cv[7].Y,cv[7].Z,MaxU,0.0F, //7
      cv[0].X,cv[0].Y,cv[0].Z,MaxU,MaxV  //0
     );

 Texture_Index+=MaxU*MaxV;
 Quad(
      cv[1].X,cv[1].Y,cv[1].Z,0.0F,MaxV, //1
      cv[6].X,cv[6].Y,cv[6].Z,0.0F,0.0F, //6
      cv[5].X,cv[5].Y,cv[5].Z,MaxU,0.0F, //5
      cv[2].X,cv[2].Y,cv[2].Z,MaxU,MaxV  //2
     );

 Texture_Index+=MaxU*MaxV;
 Quad(
      cv[7].X,cv[7].Y,cv[7].Z,0.0F,MaxV, //7
      cv[6].X,cv[6].Y,cv[6].Z,0.0F,0.0F, //6
      cv[1].X,cv[1].Y,cv[1].Z,MaxU,0.0F, //1
      cv[0].X,cv[0].Y,cv[0].Z,MaxU,MaxV  //0
     );

 Texture_Index+=MaxU*MaxV;
 Quad(
      cv[3].X,cv[3].Y,cv[3].Z,0.0F,MaxV, //3
      cv[2].X,cv[2].Y,cv[2].Z,0.0F,0.0F, //2
      cv[5].X,cv[5].Y,cv[5].Z,MaxU,0.0F, //5
      cv[4].X,cv[4].Y,cv[4].Z,MaxU,MaxV  //4
     );
}

void Process(void)
{
 int j,i=0;

 s32 Offset_Yv=0,Offset_Xv=0;

 float cosT;
 float sinT;
 float cosP;
 float sinP;
 float cosTcosP;
 float cosTsinP;
 float sinTcosP;
 float sinTsinP;

 float rotation[3] ={ 0.0F,  0.0F,  0.0F};
 float rot_speed[3]={0.02F,-0.04F,0.016F};

 while(1)
 {
  i++;

  cosT=cosf(rotation[0]);
  sinT=sinf(rotation[0]);
  cosP=cosf(rotation[1]);
  sinP=sinf(rotation[1]);

  cosTcosP=cosT*cosP;
  cosTsinP=cosT*sinP;
  sinTcosP=sinT*cosP;
  sinTsinP=sinT*sinP;

  /*
    
  x'=x;
  y':=y*cos(L)+z*sin(L) ;
  z':=-y*sin(L)+z*cos(L) ;

    Y
  x'=x*cos(L)+z*sin(L);
  y'=y;
  z'=-x*sin(L)+z*cos(L);

    Z
  x'=x*cos(L)-y*sin(L);
  y'=-x*sin(L)+y*cos(L);
  z'=z;
  */

  for(j=0;j<8;j++)
  {
   cv[j].X=      cosT*cvc[j].X+                  sinT*cvc[j].Z;
   cv[j].Y= -sinTsinP*cvc[j].X+cosP*cvc[j].Y+cosTsinP*cvc[j].Z;
   cv[j].Z=(-sinTcosP*cvc[j].X-sinP*cvc[j].Y+cosTcosP*cvc[j].Z)+sinf(i/20.0F)+1.0F;
  }

  LCD_Transfer();

  LCD_Background();

  Cube();

  Offset_Yv?Offset_Y--:Offset_Y++;

  if(Offset_Y>MaxY/3)
  {
   Offset_Yv=1;
   Offset_Y--;
  }

  if(Offset_Y<=-MaxY/3)
  {
   Offset_Yv=0;
   Offset_Y++;
  }

  Offset_Xv?Offset_X--:Offset_X++;

  if(Offset_X>MaxX/3)
  {
   Offset_Xv=1;
   Offset_X--;
  }

  if(Offset_X<=-MaxX/3)
  {
   Offset_Xv=0;
   Offset_X++;
  }

  for(j=0;j<3;++j)rotation[j]+=rot_speed[j];

#if SYNC_60_FPS
  while(!Timer_Flag);
  Timer_Flag=0;
#endif

 }
}
