#include <stdlib.h>
#include <string.h>
#include <mathf.h>

#include "Type.h"

#include "sysconfig.h"
#include "Cache.h"
#include "PSC.h"

#include "EMIFA.h"
#include "S6D04D1X21.h"

#include "EDMA3.h"
#include "Timer.h"

#include "Texture.h"
#include "Background.h"

#define SYNC_60_FPS 0 /* 1 -    (60 FPS) 0 -   FPS */

#define MaxX H
#define MaxY W

#define D 2.0F /*  */

#define MX 70.0F
#define MY 70.0F
#define OZ 2.0F

#define FRAC(t) (((float)(_spint(/*_fabsf*/(t)*256.0F)&0xFF))*0.00390625F)
#define MIX(A,B,t) ((((B)-(A))*FRAC(t))+(A))

struct TVertex
{
 float X,Y,Z;
};

float     x[2],z[2],u[2],v[2];
float     dz,du,dv;
s32 y;

s32 Offset_X=0;
s32 Offset_Y=0;

u32 *Texture_Index=NULL;

u32 SwitchBuffer[2];

float DI; //  
u8 SI;    //  

#pragma DATA_SECTION (VideoBuffer,".videobuffer")
#pragma DATA_ALIGN(VideoBuffer,32)
u16 VideoBuffer[H*W*2];

void LCD_Transfer(void)
{
 L2_WriteBack(); //   L2  

 EDMA_Wait();    //    DMA

 static u32 s=0;

 SwitchBuffer[1]=(u32)&VideoBuffer[ s    *(H*W)]; //  
 SwitchBuffer[0]=(u32)&VideoBuffer[(s^=1)*(H*W)];

 EDMA3_SRC((void*)SwitchBuffer[1]); //Turn Pointer DMA

 EDMA3_Run();                       //Run DMA
}

union tc
{
 u32 t;
 u8 c[4];
};

#pragma DATA_ALIGN(t0,4)
union tc t0;
#pragma DATA_ALIGN(t1,4)
union tc t1;
#pragma DATA_ALIGN(t2,4)
union tc t2;
#pragma DATA_ALIGN(t3,4)
union tc t3;

inline void Line(void)
{
 register s32 a=_spint(x[0]);
 register s32 b=_spint(x[1]);

 if((~(a-MaxX|b-MaxX)|a&b|y)<0)return;

 register s32 c;

 if(a>b)
 {
  c=a;
  a=b;
  b=c;
  c=1;
 }
 else c=0;

 a*=(((unsigned)a)>>31)^1; //if(a<0)a=0;

 if(b>MaxX-1)b=MaxX-1;

 if(a==b)return;

 register float xx=x[c]-(float)a;
 register float Z=z[c]-xx*dz;
 register float U=u[c]-xx*du;
 register float V=v[c]-xx*dv;

 register u32 const * restrict pt=(u32 const*)Texture_Index;
 register u16       * restrict vb=(u16*)(SwitchBuffer[0]+(((y*MaxX)+a)<<1));

 b-=a-1;

 _nassert(b>=2);
 #pragma MUST_ITERATE(2,,)
 do
 {
  register float rz=_rcpsp(Z);

  register float q=U*rz;
  register float p=V*rz;

  register u32 a0=(_spint(p)*MaxU)&((MaxU-1)*MaxU);
  register u32 a1=(a0+MaxU       )&((MaxU-1)*MaxU);
  register u32 a2= _spint(q)      &( MaxV-1)      ;
  register u32 a3=(a2+1          )&( MaxV-1)      ;

  t0.t=pt[a0+a2];
  t1.t=pt[a0+a3];
  t2.t=pt[a1+a2];
  t3.t=pt[a1+a3];

  *vb++=((_saddu4(SI,_spint(DI*MIX(MIX(t0.c[2],t2.c[2],p),MIX(t1.c[2],t3.c[2],p),q)))&0xF8)<<8)|
        ((_saddu4(SI,_spint(DI*MIX(MIX(t0.c[1],t2.c[1],p),MIX(t1.c[1],t3.c[1],p),q)))&0xFC)<<3)|
        ( _saddu4(SI,_spint(DI*MIX(MIX(t0.c[0],t2.c[0],p),MIX(t1.c[0],t3.c[0],p),q)))      >>3);

  Z=Z+dz;
  U=U+du;
  V=V+dv;
 }
 while(--b);
}

#define SORT(t,a,b) \
temp##t=a;          \
a=b;                \
b=temp##t;          \

inline float RLENGTH(struct TVertex V)
{
 return _rsqrsp((V.X*V.X)+(V.Y*V.Y)+(V.Z*V.Z));
}

inline struct TVertex NORMAL(struct TVertex V)
{
 float Length=RLENGTH(V);
 V.X*=Length;
 V.Y*=Length;
 V.Z*=Length;
 return V;
}

inline float DOT(struct TVertex V1,struct TVertex V2)
{
 return (V1.X*V2.X)+(V1.Y*V2.Y)+(V1.Z*V2.Z);
}

inline struct TVertex CROSS(struct TVertex V1,struct TVertex V2)
{
 struct TVertex Result;
 Result.X=(V1.Y*V2.Z)-(V1.Z*V2.Y);
 Result.Y=(V1.Z*V2.X)-(V1.X*V2.Z);
 Result.Z=(V1.X*V2.Y)-(V1.Y*V2.X);
 return Result;
}

void Triangle(float k0x,float k0y,float k0z,float k0u,float k0v,float k1x,float k1y,float k1z,float k1u,float k1v,float k2x,float k2y,float k2z,float k2u, float k2v)
{
 register float     ddx,ddy;
 register float     temps;
 register s32       templ;
 register s32       p0y,p1y,p2y;
 register float     p0x,p1x,p2x;
 register float     p0z,p1z,p2z;
 register float     p0u,p1u,p2u;
 register float     p0v,p1v,p2v;
 register float     dx1,dz1,du1,dv1;
 register float     dx2,dz2,du2,dv2;
 register float     dx3,dz3,du3,dv3;

 p0z=D*_rcpsp(k0z);
 p1z=D*_rcpsp(k1z);
 p2z=D*_rcpsp(k2z);

 p0x=k0x*p0z;
 p1x=k1x*p1z;
 p2x=k2x*p2z;

 register float y0=k0y*p0z;
 register float y1=k1y*p1z;
 register float y2=k2y*p2z;

 /* --------    -------- */
 if(((y2-y1)*(p2x-p0x))-((y2-y0)*(p2x-p1x))>=0.0F)return;

 /* ------------------  ------------------ */
 struct TVertex Edge0,Edge1;

 Edge0.X=(k0x-k1x)*(1.0F/MX);
 Edge0.Y=(k0y-k1y)*(1.0F/MY);
 Edge0.Z=k0z-k1z;

 Edge1.X=(k2x-k1x)*(1.0F/MX);
 Edge1.Y=(k2y-k1y)*(1.0F/MY);
 Edge1.Z=k2z-k1z;

 DI=DOT((struct TVertex){0.0F,0.0F,1.0F},/*NORMAL*/(CROSS(NORMAL(Edge0),NORMAL(Edge1))));

 if(DI<0.0F)DI=0.0F;

 SI=_spint((0.3F*256.0F)*DI*DI*DI*DI*DI*DI);

 /* ----------------------------------------------- */
 p0y=_spint(y0);
 p1y=_spint(y1);
 p2y=_spint(y2);

 p0u=k0u*p0z;
 p1u=k1u*p1z;
 p2u=k2u*p2z;

 p0v=k0v*p0z;
 p1v=k1v*p1z;
 p2v=k2v*p2z;

 p0x+=(MaxX*0.5F)+(float)Offset_X;
 p1x+=(MaxX*0.5F)+(float)Offset_X;
 p2x+=(MaxX*0.5F)+(float)Offset_X;

 p0y=(MaxY/2)-p0y-Offset_Y;
 p1y=(MaxY/2)-p1y-Offset_Y;
 p2y=(MaxY/2)-p2y-Offset_Y;

 if(p0y>p1y)
 {
  SORT(l,p0y,p1y)
  SORT(s,p0x,p1x)
  SORT(s,p0z,p1z)
  SORT(s,p0u,p1u)
  SORT(s,p0v,p1v)
 };

 if(p0y>p2y)
 {
  SORT(l,p0y,p2y)
  SORT(s,p0x,p2x)
  SORT(s,p0z,p2z)
  SORT(s,p0u,p2u)
  SORT(s,p0v,p2v)
 };

 if(p1y>p2y)
 {
  SORT(l,p1y,p2y)
  SORT(s,p1x,p2x)
  SORT(s,p1z,p2z)
  SORT(s,p1u,p2u)
  SORT(s,p1v,p2v)
 };

 if(p0y==p1y)
 {
  ddy=_rcpsp(p2y-p0y);
  dx1=(p2x-p0x)*ddy;
  dz1=(p2z-p0z)*ddy;
  du1=(p2u-p0u)*ddy;
  dv1=(p2v-p0v)*ddy;
  dx2=p1x-p0x;

  ddy=_rcpsp(p2y-p1y);
  dx3=(p2x-p1x)*ddy;
  dz3=(p2z-p1z)*ddy;
  du3=(p2u-p1u)*ddy;
  dv3=(p2v-p1v)*ddy;

  ddx=_rcpsp(dx3-dx1);
  dz=(dz3-dz1)*ddx;
  du=(du3-du1)*ddx;
  dv=(dv3-dv1)*ddx;
  if(dx1>dx2)
  {
   x[0]=p1x;
   x[1]=p0x;
   z[0]=p1z;
   z[1]=p0z;
   u[0]=p1u;
   u[1]=p0u;
   v[0]=p1v;
   v[1]=p0v;
   if(p2y>=MaxY)p2y=MaxY-1;
   for(y=p1y;y<=p2y;y++)
   {
    Line();
    x[0]=x[0]+dx3;
    x[1]=x[1]+dx1;
    z[0]=z[0]+dz3;
    z[1]=z[1]+dz1;
    u[0]=u[0]+du3;
    u[1]=u[1]+du1;
    v[0]=v[0]+dv3;
    v[1]=v[1]+dv1;
   };
  }
  else
  {
   x[0]=p0x;
   x[1]=p1x;
   z[0]=p0z;
   z[1]=p1z;
   u[0]=p0u;
   u[1]=p1u;
   v[0]=p0v;
   v[1]=p1v;
   if(p2y>=MaxY)p2y=MaxY-1;
   for(y=p1y;y<=p2y;y++)
   {
    Line();
    x[0]=x[0]+dx1;
    x[1]=x[1]+dx3;
    z[0]=z[0]+dz1;
    z[1]=z[1]+dz3;
    u[0]=u[0]+du1;
    u[1]=u[1]+du3;
    v[0]=v[0]+dv1;
    v[1]=v[1]+dv3;
   };
  };
 }
 else if(p1y==p2y)
 {
  ddy=_rcpsp(p2y-p0y);
  dx1=(p2x-p0x)*ddy;
  dz1=(p2z-p0z)*ddy;
  du1=(p2u-p0u)*ddy;
  dv1=(p2v-p0v)*ddy;

  ddy=_rcpsp(p1y-p0y);
  dx2=(p1x-p0x)*ddy;
  dz2=(p1z-p0z)*ddy;
  du2=(p1u-p0u)*ddy;
  dv2=(p1v-p0v)*ddy;

  ddx=_rcpsp(dx2-dx1);
  dz=(dz2-dz1)*ddx;
  du=(du2-du1)*ddx;
  dv=(dv2-dv1)*ddx;
  x[0]=p0x;
  x[1]=p0x;
  z[0]=p0z;
  z[1]=p0z;
  u[0]=p0u;
  u[1]=p0u;
  v[0]=p0v;
  v[1]=p0v;
  if(dx1>dx2)
  {
   if(p1y>=MaxY)p1y=MaxY-1;
   for(y=p0y;y<=p1y;y++)
   {
    Line();
    x[0]=x[0]+dx2;
    x[1]=x[1]+dx1;
    z[0]=z[0]+dz2;
    z[1]=z[1]+dz1;
    u[0]=u[0]+du2;
    u[1]=u[1]+du1;
    v[0]=v[0]+dv2;
    v[1]=v[1]+dv1;
   }
  }
  else
  {
   if(p1y>=MaxY)p1y=MaxY-1;
   for(y=p0y;y<=p1y;y++)
   {
    Line();
    x[0]=x[0]+dx1;
    x[1]=x[1]+dx2;
    z[0]=z[0]+dz1;
    z[1]=z[1]+dz2;
    u[0]=u[0]+du1;
    u[1]=u[1]+du2;
    v[0]=v[0]+dv1;
    v[1]=v[1]+dv2;
   }
  }
 }
 else
 {
  ddy=_rcpsp(p2y-p0y);
  dx1=(p2x-p0x)*ddy;
  dz1=(p2z-p0z)*ddy;
  du1=(p2u-p0u)*ddy;
  dv1=(p2v-p0v)*ddy;

  ddy=_rcpsp(p1y-p0y);
  dx2=(p1x-p0x)*ddy;
  dz2=(p1z-p0z)*ddy;
  du2=(p1u-p0u)*ddy;
  dv2=(p1v-p0v)*ddy;

  ddy=_rcpsp(p2y-p1y);
  dx3=(p2x-p1x)*ddy;
  dz3=(p2z-p1z)*ddy;
  du3=(p2u-p1u)*ddy;
  dv3=(p2v-p1v)*ddy;

  ddx=_rcpsp(dx2-dx1);
  dz=(dz2-dz1)*ddx;
  du=(du2-du1)*ddx;
  dv=(dv2-dv1)*ddx;
  x[0]=p0x;
  x[1]=p0x;
  z[0]=p0z;
  z[1]=p0z;
  u[0]=p0u;
  u[1]=p0u;
  v[0]=p0v;
  v[1]=p0v;
  if(dx1>dx2)
  {
   if(p1y>=MaxY)p1y=MaxY-1;
   for(y=p0y;y<p1y;y++)
   {
    Line();
    x[0]=x[0]+dx2;
    x[1]=x[1]+dx1;
    z[0]=z[0]+dz2;
    z[1]=z[1]+dz1;
    u[0]=u[0]+du2;
    u[1]=u[1]+du1;
    v[0]=v[0]+dv2;
    v[1]=v[1]+dv1;
   };
   if(p2y>=MaxY)p2y=MaxY-1;
   for(y=p1y;y<=p2y;y++)
   {
    Line();
    x[0]=x[0]+dx3;
    x[1]=x[1]+dx1;
    z[0]=z[0]+dz3;
    z[1]=z[1]+dz1;
    u[0]=u[0]+du3;
    u[1]=u[1]+du1;
    v[0]=v[0]+dv3;
    v[1]=v[1]+dv1;
   };
  }
  else
  {
   if(p1y>=MaxY)p1y=MaxY-1;
   for(y=p0y;y<p1y;y++)
   {
    Line();
    x[0]=x[0]+dx1;
    x[1]=x[1]+dx2;
    z[0]=z[0]+dz1;
    z[1]=z[1]+dz2;
    u[0]=u[0]+du1;
    u[1]=u[1]+du2;
    v[0]=v[0]+dv1;
    v[1]=v[1]+dv2;
   };
   if(p2y>=MaxY)p2y=MaxY-1;
   for(y=p1y;y<=p2y;y++)
   {
    Line();
    x[0]=x[0]+dx1;
    x[1]=x[1]+dx3;
    z[0]=z[0]+dz1;
    z[1]=z[1]+dz3;
    u[0]=u[0]+du1;
    u[1]=u[1]+du3;
    v[0]=v[0]+dv1;
    v[1]=v[1]+dv3;
   };
  };
 };
}

#include "Cube.inc"

void main(void)
{
 L1P_Config(L1_SIZE_32K);         //Enable L1P L1D L2 Cache
 L1D_Config(L1_SIZE_32K);
 L2_Config(L2_SIZE_128K);         //Use 50% only, rest for Program

 SDRAM_Cacheable(CACHEABLE_ON);   //Enable SDRAM Cacheable

 KICK0R=0x83E70B13;               //Unlock
 KICK1R=0x95A4F1E0;

 PINMUX10=0x00000080;             //GP3[0] => LCD_RST

 PINMUX13=0x11000000;             //EMIFA
 PINMUX14=0x00111111;
 PINMUX17=0x00000100;
 PINMUX18=0x00101010;

 CFGCHIP0=(2<<2)|2;               //PLL MMRs unlock & TC1 64 byte & TC0 64 byte (DBS)

 PSC0_lPSC_enable(0,0);           //enable EDMA3 Channel Controller
 PSC0_lPSC_enable(0,1);           //enable EDMA3 Transfer Controller 0

 PSC0_lPSC_enable(0,3);           //enable EMIFA

 PSC1_lPSC_enable(0,3);	          //enable GPIO

 EMIFA_Init();

 /*
 0 Round toward nearest representable floating-point number
 1 Round toward 0 (truncate)
 2 Round toward infinity (round up)
 3 Round toward negative infinity (round down)
 */
 FADCR&=~((3<<25)|(3<<9));
 FADCR|=(1<<25)|(1<<9);           //truncate mode

 LCD_Reset();
 LCD_Init();
 LCD_Clear(0);

 LCD_Position(0,0,H-1,W-1);

 EDMA3_Init((void*)0,W*H*2);      //Init EDMA3

#if SYNC_60_FPS
 Timer_INT_Init(16);              //16ms ~60 Hz
#endif

 Process();
}
